<?php if (!defined('PmWiki')) exit();

/*
  Mailform4 (alias Mailform2²) by Markus Konrad <mako@mako-home.de>
  PmWiki module for mail forms.
  More information at http://www.pmwiki.org/wiki/Cookbook/Mailform4
  
  Based on Mailform2, created by Joachim Durchholz (jo@durchholz.org).
  See http://www.pmwiki.org/wiki/Cookbook/Mailform2
  
  Contribution by Antony Templier <antony.yemplier@gmail.com>
  -Minor update to allow full translation
  -French translation
  
  This code is licensed under the
  GNU General Public License version 2.0
  as distributed with PmWiki.
*/

$RecipeInfo['Mailform4']['Version'] = '2012-01-11';

Markup('mailform4-show-msgs', 'directives', "/\\(:mailform4-show-msgs:\\)/e", "Mailform4ShowMsgs()");

/******************************************************************************/
/* DEFAULT VARIABLE VALUES                                                    */
/******************************************************************************/

SDVA(
  $Mailform4,
  array(
    'recipient' => '',
    'subject' => 'contact form message',
    'mailform4_url' => 'http://www.pmwiki.org/wiki/Cookbook/Mailform4',
    'website' => $ScriptUrl,
    'ip' => $_SERVER['REMOTE_ADDR'],
    'host' => gethostbyaddr($_SERVER['REMOTE_ADDR']),
    'datetime' => date('Y-m-d H:i:s')
  )
);


SDVA($Mailform4ValidFields,
  array(
    'name' => array('name', 3, 120),
    'sender' => array('email', 5, 120),
    'text' => array('string', 3, null)
  )
);


SDV($Mailform4Disabled, 0);

SDV($Mailform4MailTemplate, 'Mailform4/MailTemplate');
SDV($Mailform4FormTemplate, 'Mailform4/FormTemplate');

SDV($HandleActions['mailform4'], 'Mailform4Handler');


/******************************************************************************/
/* MAIN CODE                                                                  */
/******************************************************************************/

if ($Lang == 'de' || $XLLangs[0] = 'de')
  XLPage('de','Site.XLPage-Mailform4-de');

if ($Lang == 'fr' || $XLLangs[0] = 'fr')
  XLPage('fr','Site.XLPage-Mailform4-fr');


if (!isset($_SESSION['Mailform4']))
{
  $_SESSION['Mailform4'] = array('errors' => null, 'info' => null);
}

$Mailform4Msgs = &$_SESSION['Mailform4'];


Mailform4SetDefaultPVs();

function Mailform4SetDefaultPVs()
{
  global $Mailform4, $FmtPV;

  foreach ($Mailform4 as $field => $val)
  {
    $FmtPV['$Mailform4' . ucfirst($field)] = '"' . $val . '"';
  }
}

function Mailform4Sanitise($str)
{
  return preg_replace ('[\\0-\\37]', '', stripmagic($str));
}

function Mailform4ValueIsEmail($field, $v, $min, $max)
{
  global $Mailform4Msgs;
  
  $ok = Mailform4ValueIsString($field, $v, $min, $max);
	$ok = $ok && preg_match("/^[a-zA-Z0-9]{1}[_a-zA-Z0-9-]*(\.[_a-zA-Z0-9-]+)*@([a-zA-Z0-9-\.]+)*([a-zA-Z0-9-]{2,})\.[a-zA-Z]{2,}$/", $v);
	
	if (!$ok)
	  $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . XL('Please enter a valid email address.');
	
	return $ok;
}

function Mailform4ValueIsNumber($field, $v, $min, $max)
{
  global $Mailform4Msgs;
  
  $ok = is_numeric($v);
  
  if ($min !== null)
    $ok = $ok && ($v >= $min);
  if ($max !== null)
    $ok = $ok && ($v <= $max);

	if (!$ok)
	  $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please enter a valid number between %d and %d.'), $min, $max);
	
	return $ok;
}

function Mailform4ValueIsString($field, $v, $min, $max)
{
  global $Mailform4Msgs;
  
  $len = strlen($v);
  if ($min !== null)
    $ok = ($len >= $min);
  if ($max !== null)
    $ok = $ok && ($len <= $max);
  
	if (!$ok)
	  $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please enter a string that is between %d and %d characters long.'), $min, $max);

  return $ok;
}

function Mailform4ValueIsCheckbox($field, $v, $min, $max)
{
  global $Mailform4Msgs;
  
  if ($min === FALSE && $max === FALSE)
  {
    $ok = ($v == "unchecked");
    if (!$ok)
      $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please uncheck the checkbox.'));
  }
  else if ($min === TRUE && $max === TRUE)
  {
    $ok = ($v == "checked");
    if (!$ok)
      $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please check the checkbox.'));
  }
  else
  {
    $ok = ($v == "checked" || $v == "unchecked");
    if (!$ok)
      $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please fix the checkbox.'));
  }

  return $ok;
}

function Mailform4ValueIsRadio($field, $v, $min, $max)
{
  global $Mailform4Msgs;
  
  if (is_array($min) && in_array($v, $min))
    $ok = TRUE;
  else
  {
    $ok = FALSE;
    $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . sprintf(XL('Please select one radio button.'));
  }

  return $ok;
}

function Mailform4Handler($pagename)
{
  global $Mailform4, $Mailform4ValidFields;
  global $Mailform4Disabled, $Mailform4MailTemplate;
  global $Mailform4Msgs;
  global $EnablePostCaptchaRequired, $ScriptUrl, $FmtPV;

  if ($Mailform4Disabled)
  {
    $Mailform4Msgs['error'][] = XL('Sending mails has been disabled.');
  }
  else if (isset($_GET['success']) && $_GET['success'] == "1")
  {
    $Mailform4Msgs['info'][] = XL('Thank you! Your mail has been successfully sent!');
  }
  else
  {
    $error = false;
    
    $fields = array_merge($Mailform4, $Mailform4ValidFields);
    $submittedValues = array();
    foreach ($fields as $field => $check)
    {
      if (isset($Mailform4ValidFields[$field]))
      {
	## Try to Validate Data
	if ($check[0] == 'checkbox')
	{
	  $submitted = array_key_exists($field, $_POST['mailform4'])? 'checked': 'unchecked';
	}
        else if (isset($_POST['mailform4'][$field]) && strlen(trim($_POST['mailform4'][$field])) > 0)
        {
          $submitted = trim(Mailform4Sanitise($_POST['mailform4'][$field]));
        }
        else
        {
          if ($check[1] != null)
          {
            $Mailform4Msgs['error'][] = ucfirst(XL($field)) . ': ' . XL('Please enter a value.');
            $error = true;
          }
          
          $submitted = '';
        }
        
        if ($submitted != '')
        {
          $chkFunc = 'Mailform4ValueIs' . ucfirst($check[0]);
          $error = $error || !($chkFunc($field, $submitted, $check[1], $check[2]));
        }
        
	$val = $submitted;

	## Quote carefully to avoid injection attacks
        switch ($check[0])
	{
	  case 'string':
	    $FmtPV['$Mailform4' . ucfirst($field)] = '"[=' . $val . '=]"';
	    break;
	  case 'checkbox':
	    $FmtPV['$Mailform4' . ucfirst($field)] = ($val == 'checked')? $val: '';
	    break;
	  case 'radio':
	    foreach ($check[1] as $opt)
	      if ($opt == $val)
		$FmtPV['$Mailform4' . ucfirst($field) . ucfirst($opt)] = 'checked';
	    break;
	  default:
	    $FmtPV['$Mailform4' . ucfirst($field)] = '"' . $val . '"';
	    break;
	}
      }
      else
      {
	$val = $check;
	$FmtPV['$Mailform4' . ucfirst($field)] = '"' . $val . '"';
      }
      
      $submittedValues[$field] = $val;
    }
    
    
    if ($EnablePostCaptchaRequired && !IsCaptcha())
    {
      $Mailform4Msgs['error'][] = XL('Invalid CAPTCHA code. Please try again.');
      $error = true;
    }
    
    if (!$error)
    {
      $msgTpl = ReadPage($Mailform4MailTemplate);
      $msgTpl = $msgTpl['text'];
      foreach ($submittedValues as $field => $val)
      {
        $msgTpl = str_replace('%' . strtoupper($field) . '%', $val, $msgTpl);
      }
      
      if (!mail(
          $Mailform4['recipient'],
          $Mailform4['subject'],
          $msgTpl,
          'From: ' . trim($FmtPV['$Mailform4Sender'],'"') ))
      {
        $Mailform4Msgs['error'][] = XL('Failure sending email.');
      }
      else
      {
        Redirect($pagename, '$PageUrl?action=mailform4&success=1');
        exit;
      }
    }
  }
  
  $GLOBALS['action'] = 'browse';
  HandleDispatch($pagename, $GLOBALS['action']);
}

/**
 * Display error- or success-messages that have been saved in the session
 */
function Mailform4ShowMsgs()
{
  global $Mailform4Msgs;
  
	$out = '';
	  
	if (isset($Mailform4Msgs['error']))
	{
  	//var_dump($Mailform4Msgs['error']);exit;
	  $out .= Mailform4ShowMsgType('error', $Mailform4Msgs['error']);
	}
	  
	if (isset($Mailform4Msgs['info']))
	{
	  //var_dump($Mailform4Msgs['info']);exit;
	  $out .= Mailform4ShowMsgType('info', $Mailform4Msgs['info']);
	}
	  
	return Keep($out);
}


function Mailform4ShowMsgType($type, &$msgs)
{
  $out = '';
	  
  if (is_array($msgs) && count($msgs) > 0)
  {
    $out .= '<div class="' . $type . '_msg">';
	    
    foreach ($msgs as $m)
    {
      $out .= $m . '<br />';
    }
	    
    $out .= '</div>';
  }
	  
  $msgs = null;
  
  return $out;
}
